X(LVTTL33)
X(LVCMOS33)
X(LVCMOS25)
X(LVCMOS18)
X(LVCMOS15)
X(LVCMOS12)

X(SSTL18_I)
X(SSTL18_II)
X(SSTL15_I)
X(SSTL15_II)
X(SSTL135_I)
X(SSTL135_II)
X(HSUL12)

X(SSTL18D_I)
X(SSTL18D_II)
X(SSTL135D_I)
X(SSTL135D_II)
X(SSTL15D_I)
X(SSTL15D_II)
X(HSUL12D)
X(LVCMOS33D)
X(LVCMOS25D)
X(LVCMOS15D)
X(LVCMOS12D)

X(LVDS)
X(BLVDS25)
X(MLVDS25)
X(LVPECL33)
X(SLVS)
X(SUBLVDS)
X(LVCMOS18D)

X(LVDS25E)
X(BLVDS25E)
X(MLVDS25E)
X(LVPECL33E)
